class InvalidNodeException(Exception): ...


class DialectValidationError(Exception): ...


class InvalidColumnNameException(Exception): ...


class InvalidColumnsException(Exception): ...


class InvalidTableNameException(Exception): ...


class InvalidSchemaNameException(Exception): ...


class InvalidJoinException(Exception): ...
