from typing import runtime_checkable, Protocol, Any


@runtime_checkable
class DialectProtocol(Protocol):
    def validate(self, current_node, next_node) -> None: ...

    def escape_identifier(self, identifier: str) -> str: ...
    def validate_identifier(self, identifier: str) -> str: ...
    def normalize_identifiers_args(self, args: Any) -> str: ...
